#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>
#include <stdlib.h>
#include <inttypes.h>
#include "functions.h"
#include "variables.h"
#include "defines.h"
#include "baud_calc.h"
#include "debounce.h"
#include "uart_init_getc.h"

void ME37047009(void)
{

uint8_t ERR;
uint8_t ERR_FLAG;
double temp1;
double temp2;
ADC_init();
uart_init();	 

while(1)
	{
	    if (debounce(PINB,D_IN0)) //if button on D_IN0 is pushed
        {

			ERR=0;
			ERR_FLAG=0;

			
			check_ADC(&temp1, 3, 2.38, 2.6);	// 12V Zuleitung; equals 11.8V to 12.3V
#ifdef DEBUG
	EIA_232_debug_ADC("12 V Zuleitung", temp1);
	uart_puts("2,38V = 11,8 V; 2,6V =12,3V\n");
#endif
	EIA_232_error_ADC("12V Zuleitung(2,4V = 11,8 V; 2,6V = 12,3V)", temp1);//
			
/*************************** Durchgangsprfung ***************************/
			
			PORTD |= (1<<D_OUT1);					//relay: Zg
			_delay_ms(150);

			check_con_passage(S0|S1|S2,D_IN1);		// D_IN1: 12V_L to 0V_L; H | H | H = Y7
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_L","0V_L(D_IN1)");	//
#endif												//
	EIA_232_error_durchgang("12V_L","0V_L(D_IN1)");	//


			check_con_passage(S1|S2,D_IN2);			// D_IN2: 12V_L to 0V_L; L | H | H = Y6
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_L","0V_L(D_IN2)");	//
#endif												//
	EIA_232_error_durchgang("12V_L","0V_L(D_IN2)");	//


			check_con_passage(S0|S2,D_IN3);			// D_IN3: 12V_L to 0V_L; H | L | H = Y5
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_L","0V_L(D_IN3)");	//
#endif												//
	EIA_232_error_durchgang("12V_L","0V_L(D_IN3)");	//


			check_con_passage(S2,D_IN4);			// D_IN4: 12V_L to 0V_L; L | L | H = Y4
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_L","0V_L(D_IN4)");	//
#endif												//
	EIA_232_error_durchgang("12V_L","0V_L(D_IN4)");	//


			if (!(PINB & (1<<D_IN5)))				// D_IN5: 12V_L to 0V_L
				ERR += 1;							//
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_L","0V_L(D_IN5)");	//
#endif												//
	EIA_232_error_durchgang("12V_L","0V_L(D_IN5)"); //


			if (!(PINB & (1<<D_IN6)))				// D_IN6: 12V_L to 0V_L 
				ERR += 1;							//
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_L","0V_L(D_IN6)");	//
#endif												//
	EIA_232_error_durchgang("12V_L","0V_L(D_IN6)");	//


			if (!(PINB & (1<<D_IN7)))				// D_IN7: 12V_E to 0V_E
				ERR += 1;							//
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_E", "0V_E");		//
#endif												//
	EIA_232_error_durchgang("12V_E","0V_E");		//


			check_con_passage(S0|S1,D_IN8);			// D_IN8: CAN_EN to CAN_GND; H | H | L = Y3
#ifdef DEBUG										//
	EIA_232_debug_durchgang("CAN_EN", "CAN_GND");	//
#endif												//
	EIA_232_error_durchgang("CAN_EN","CAN_GND");	//
			
			
			PORTD |= (1<<D_OUT2); 					// relay: CAN-Verbindung
			_delay_ms(50);						
			check_con_passage(S1,D_IN9); 			// D_IN9: T_CAN_H_OUT to GND; L | H | L = Y2
			PORTD &=~ (1<<D_OUT2); 					// relay: CAN-Verbindung
			_delay_ms(100);							//
#ifdef DEBUG										//
	EIA_232_debug_durchgang("T_CAN_H_OUT", "GND");	//
#endif												//
	EIA_232_error_durchgang("T_CAN_H_OUT", "GND");	//


			check_con_passage(S0,D_IN10); 			// D_IN10: T_12VE to T_GNDE & T_CAN_GND; H | L | L = Y1
#ifdef DEBUG										//
	EIA_232_debug_durchgang("12V_E", "T_CAN_GND");	//
#endif												//
	EIA_232_error_durchgang("12V_E", "T_CAN_GND"); 	//
			
						
			PORTB |= (1<<D_OUT4);					// relay: K
			_delay_ms(100);							//
			check_con_passage(0x00,D_IN11);			// D_IN11: K to 0V_E; L | L | L = Y0
			PORTB &=~ (1<<D_OUT4);					// relay: K
#ifdef DEBUG										//
	EIA_232_debug_durchgang("K", "0V_E");		//
#endif												//
	EIA_232_error_durchgang("K", "0V_E"); 


			PORTB |= (1<<D_OUT5);					//relay: Z
			_delay_ms(100);							//
		    check_con_passage(S1,D_IN12);			// D_IN12: Z to 0V_E; L | H | L = Y2
			PORTB &=~ (1<<D_OUT5);					//relay: Z
#ifdef DEBUG										//
	EIA_232_debug_durchgang("Z", "0V_E");	//
#endif												//
	EIA_232_error_durchgang("Z", "0V_E");	//
/*************************** Durchgangsprfung ***************************/


/********************************** ADC **********************************/


			check_ADC(&temp1, ADC0, 2.3, 2.7);		//ADC_0: Pegel CAN H without load
			_delay_ms(100);							//
#ifdef DEBUG										//
	EIA_232_debug_ADC("Pegel CAN H", temp1);		//
#endif												//
	EIA_232_error_ADC("Pegel CAN H", temp1);		//


			check_ADC(&temp2, ADC1, 2.3, 2.7);		//ADC_1: Pegel CAN L without load
			_delay_ms(100);							//
#ifdef DEBUG										//
	EIA_232_debug_ADC("Pegel CAN L", temp2);		//
#endif												//
	EIA_232_error_ADC("Pegel CAN L", temp2);		//	


			isEqual(temp1,temp2,0.1);				// if difference is larger than 0.1
			if (ERR>0)								//
			{										//
				uart_puts("Spannungsunterschied zwischen CAN H und CAN L grer als 0,1V\n\n");
				ERR=0;								//
			}										//


													//ADC_0: Pegel CAN H (load + 5V)
			PORTD |= (1<<D_OUT3);					//relay: load + 5V
			_delay_ms(150);							//
			check_ADC(&temp1, ADC0, 3.55, 3.95);	//
			PORTD &=~ (1<<D_OUT3);					//relay: load + 5V
#ifdef DEBUG										//
	EIA_232_debug_ADC("Pegel CAN H + Last,5V",temp1);//
#endif												//
	EIA_232_error_ADC("Pegel CAN H + Last,5V",temp1);//


													//ADC_1: Pegel CAN L (load + GND)
			PORTB |= (1<<D_OUT6);					//relay: load + GND
			_delay_ms(100);							//
			check_ADC(&temp2, ADC1, 1.05, 1.85);	//
			PORTB &=~ (1<<D_OUT6);					//relay: load + GND
#ifdef DEBUG										//
	EIA_232_debug_ADC("Pegel CAN L + Last, GND",temp2);//
#endif												//
	EIA_232_error_ADC("Pegel CAN L + Last,GND", temp2);//


													// ADC_2: Diode K/Z
													//
			check_ADC(&temp1, ADC3, 2.38, 2.6);		// 12V Zuleitung; equals 11.8V to 12.3V
	EIA_232_error_ADC("12V Zuleitung(2,38V = 11,8 V; 2,6V = 12,3V)", temp1);//


			check_ADC(&temp2, ADC2, 2.25, 2.45);	// C-13 (12V_E 8 pol. Buchse)
#ifdef DEBUG										//
	EIA_232_debug_ADC("Diodenkabel (8 pol. Buchse)", temp2);//
#endif												//

	

			if((temp1-temp2)<0.08)					// if voltage 
				uart_puts("\nDiode nicht vorhanden oder falsch verbaut");//								
			_delay_ms(100);							//

/********************************** ADC **********************************/

			if(ERR_FLAG>0)							// if error detected
	    	{
				PORTD |=(1<<D_OUT0); 				//LED red
				_delay_ms(10000);
			}
			else
				uart_puts("\n\nAlles in Ordnung\n\n");
		}
		else 										// if button is not pushed down
		{
			PORTD &=~ (1<<D_OUT0);					//LED green
			PORTD &=~ (1<<D_OUT1);					//relay: Zg off
		} 
	}	
}
