#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>
#include <stdlib.h>
#include <inttypes.h>
#include "functions.h"
#include "variables.h"
#include "baud_calc.h"

uint8_t ERR;
uint8_t ERR_FLAG;

/************* functions for ADC & connection passages *************/
// initialize ADC
void ADC_init(void)
{
	uint16_t result;	

	ADMUX = (1<<REFS0);		// reference voltage = AVCC (5V)
	ADCSRA |= (1<<ADEN);		//activate ADC

//"dummy-readout"
	ADCSRA |= (1<<ADSC);				
	while(ADCSRA & (1<<ADSC));
	result = ADCW;
}


double ADC_read(uint8_t channel)
{
	// Function gets voltage at given ADC-PORT
	// and returns the value in volts.
	//
	ADMUX = (ADMUX &=~(0x1F)) | (channel & 0x1F);// PORT of multiplexer (choose channel)
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
	ADCSRA |= (1<<ADSC);						// single conversion
	while(ADCSRA &(1 << ADSC)); 				// wait for result
	return (ADCW/1023.0)*5.0;					// return result
}

void isEqual(double a, double b, double allowance)
{
	//function gets two values and compares

	double t;

	if(a > b)			//check for absolute value
		t = a - b;
	else
		t = b - a;	

	if(t>allowance)
	{
		ERR  += 1;
		ERR_FLAG += 1;
	}
}

void check_con_passage(uint8_t pins, uint8_t D_INX)
{
	//function checks the connecting passage and sets error flag if necessary
	PORTC |= pins;	//MUX port
	_delay_ms(50);

	if(!(PINB & 1<<D_INX))
	{
		ERR  += 1;
		ERR_FLAG += 1;
	}

   PORTC =0x00;		//reset MUX
   _delay_ms(50);
}

void check_ADC(double *temp_x, uint8_t port, double left, double right)
{
		*temp_x = ADC_read(port);
		_delay_ms(100);

		if((*temp_x < left ) || (*temp_x > right))
		{
			ERR  += 1;
			ERR_FLAG += 1;
		}
}
/********************************** functions for UART **********************************/

/*initialize UART*/
void uart_init(void)
{

		UCSR0B |= (1<<TXEN0)|(1<<RXEN0);  // enable UART TX and RX (transmit and receive)
		UCSR0C |= (1<<UCSZ01)|(1<<UCSZ00)|(1<<USBS0);  // asynchron 8N1 
 
		UBRR0H = UBRR_VAL >> 8;
	 	UBRR0L = UBRR_VAL & 0xFF;
}

/* char output */
int uart_putc(unsigned char c)
{
    while (!(UCSR0A & (1<<UDRE0)))  /* hold on until sendprocess available */
    {
    }                             
 
    UDR0 = c;                      /* send char*/
    return 0;
}
 
/* string output   ///  puts is independent from conroller */
void uart_puts (char *s)
{
    while (*s)
    {   /* takes as long as *s != '\0' disparate to the "string-endchar" */
        uart_putc(*s);
        s++;
    }
} 

/* char input */
uint8_t uart_getc(void)
{
    while (!(UCSR0A & (1<<RXC0)))   // hold on until char available
        ;
    return UDR0;                   // return char
}

void EIA_232_error_durchgang(char *connection1, char *connection2)
{
	if(ERR>0)						
	{	
		uart_puts("Kein Durchgang von Leitung ");
		uart_puts(connection1);
		uart_puts(" zu Leitung ");
		uart_puts(connection2);
		uart_puts("\n\n");
	}
	ERR=0;

}

void EIA_232_error_ADC(char *connection1, double value)
{
	char number[8];
	if(ERR>0)
	{
		uart_puts("\nSpannungswert an ");
		uart_puts(connection1);
		uart_puts(" mit einem Wert von ");
		dtostrf(value, 6, 2, number );			//number output /** include libm.a: Project -> Configuration Options -> Libaries -> libm.a; include with arrow to the right **/
		uart_puts(number);
		uart_puts("V nicht korrekt\n");
	}
	ERR=0;
}

void EIA_232_debug_ADC(char *connection,double value)
{
	if(ERR==0)
	{
		char number[8];
		uart_puts("\n\nSpannung an ");
		uart_puts(connection);
		uart_puts(" = ");
		dtostrf(value, 6, 2, number );
		uart_puts(number);	
	}
}

void EIA_232_debug_durchgang(char *connection1, char *connection2)
{	
	if(ERR==0)
	{
		uart_puts("\nDurchgang besteht von Leitung ");
		uart_puts(connection1);
		uart_puts(" zu Leitung ");
		uart_puts(connection2);
		uart_puts("\n");
	}
}
